# -*- coding: utf-8 -*-

from imio import history as imio_history
from imio.history.config import HISTORY_COMMENT_NOT_VIEWABLE
from imio.history.testing import IntegrationTestCase
from plone import api
from plone.app.testing import TEST_USER_NAME
from Products.Five import zcml
from zope.component import getMultiAdapter


class TestContentHistory(IntegrationTestCase):

    def setUp(self):
        super(TestContentHistory, self).setUp()
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        self.doc = doc

    def test_getHistory(self):
        """Test the getHistory method.
           It should return the history of the current object workflow
           in reverse order, last event first."""
        # create a document
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        # we can get the workflow history even when no transition was triggered
        history = view.getHistory()
        # this is the 'element created' event in the history
        self.assertEqual(len(history), 2)
        self.assertEqual(history[0]['action'], 'Edited')
        self.assertEqual(history[0]['type'], 'versioning')
        self.assertEqual(history[0]['actor'], TEST_USER_NAME)
        self.assertIsNone(history[1]['action'])
        self.assertEqual(history[1]['type'], 'workflow')
        # publish the doc
        self.wft.doActionFor(self.doc, 'publish')
        history = view.getHistory()
        # actions are sorted reverse so first element of history is still last action
        self.assertEqual(len(history), 3)
        self.assertEqual(history[0]['action'], 'publish')

        # when changing an element workflow, getHistory will return events
        # of the currently applied workflow
        self.wft.setChainForPortalTypes(('Document', ), ('intranet_workflow',))
        # when we changed the workflow for an element, it still does not changed
        # the workflow_history so change workflow state for it to happen
        self.wft.doActionFor(self.doc, 'hide')
        # now using getHistory will return right history
        history = view.getHistory()
        self.assertEqual(history[0]['action'], 'hide')

        # test that it works if element has no more workflow defined in portal_workflow
        # define no workflow anymore for 'Document'
        self.wft.setChainForPortalTypes(('Document', ), [])
        for event in view.getHistory():
            self.assertNotEqual(event['type'], 'workflow')

        # test that it works also with an element having no workflow
        # and so no workflow_history attribute
        # we will test with type 'File' that does not have a workflow by default
        fileType = self.portal.portal_types['File']
        self.assertEqual(self.wft.getWorkflowsFor(fileType), [])
        afile = api.content.create(type='File',
                                   id='afile',
                                   container=self.portal)
        view = getMultiAdapter((afile, self.portal.REQUEST), name='contenthistory')
        # no workflow_history attribute
        self.assertTrue(not hasattr(afile, 'workflow_history'))
        # this does not fail
        self.assertEqual(view.getTransitionTitle('a_transition'), 'a_transition')
        self.assertEqual(view.getHistory(), [])

    def test_getTransitionTitle(self):
        """Test the getTransitionTitle method.
           This will return the title of a transition if it has one, the id otherwise."""
        # create a document
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        # it use the 'simple_publication_workflow'
        # test with an existing transition
        self.assertEqual(view.getTransitionTitle('publish'),
                         'Reviewer publishes content')
        # if the transition does not exist, it will return passed transition id
        self.assertEqual(view.getTransitionTitle('unexisting_transition_id'),
                         'unexisting_transition_id')
        # if a transition does not have a title, the passed transition id is returned
        self.wft.simple_publication_workflow.transitions['publish'].title = ''
        self.assertEqual(view.getTransitionTitle('publish'),
                         'publish')
        # an entry that is not a transition_id but contains special chars does not break
        self.assertEqual(view.getTransitionTitle(u'sp\xe9cial'),
                         u'sp\xe9cial')
        self.assertEqual(view.getTransitionTitle('sp\xc3\xa9cial'),
                         'sp\xc3\xa9cial')
        # empty value does not break
        self.assertEqual(view.getTransitionTitle(''), '')

    def test_showColors(self):
        """Test the showColors method.
           This is defined to be easily overrided, for now it is always True."""
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        self.assertTrue(view.showColors())

    def test_MayViewComment(self, ):
        """Test the mayViewComment method.
           We will register an adapter that hide 'publish' transition comments."""
        # by default, mayViewComment returns "True" so every comments
        # are viewable in the object's history
        # create a document and publish it, the comment of the 'publish' transition is viewable
        self.wft.doActionFor(self.doc, 'publish', comment='My comment')
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        history = view.getHistory()
        lastEvent = history[0]
        self.assertEqual(lastEvent['action'], 'publish')
        self.assertEqual(lastEvent['comments'], 'My comment')

        # now register an adapter that will do 'publish' transition comment not visible
        zcml.load_config('testing-adapter.zcml', imio_history)
        self.request.set('hide_wf_history_comment', True)
        history = view.getHistory()
        lastEvent = history[0]
        self.assertEqual(lastEvent['action'], 'publish')
        # now comment is no more viewable
        self.assertEqual(lastEvent['comments'], HISTORY_COMMENT_NOT_VIEWABLE)

        # getHistory can be called with checkMayViewComment set to False,
        # in this case, mayViewComment check is not done
        history = view.getHistory(checkMayViewComment=False)
        lastEvent = history[0]
        self.assertEqual(lastEvent['action'], 'publish')
        # comment is viewable as mayViewComment was not done
        self.assertEqual(lastEvent['comments'], 'My comment')
        # cleanUp zmcl.load_config because it impact other tests
        zcml.cleanUp()

    def test_MayViewEvent(self):
        """Test the mayViewEvent method.
           We will register an adapter that hides the 'publish' event."""
        # by default, mayViewEvent returns "True" so every events
        # are viewable in the object's history
        # create a document and publish it, 2 events are viewable
        self.wft.doActionFor(self.doc, 'publish', comment='My comment')
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        history = view.getHistory()
        self.assertEqual(len(history), 3)
        actions = [event['action'] for event in history]
        self.assertTrue('publish' in actions)
        # we have also a revision
        self.assertTrue(u'Edited' in actions)

        # enable testing-adapter.zcml, the 'publish' actions will
        # not be viewable anymore, as well as revisions
        zcml.load_config('testing-adapter.zcml', imio_history)
        self.request.set('hide_wf_history_event', True)
        self.request.set('hide_revisions_event', True)
        history = view.getHistory()
        self.assertEqual(len(history), 1)
        actions = [event['action'] for event in history]
        self.assertFalse('publish' in actions)
        self.assertFalse(u'Edited' in actions)

        # if passing checkMayViewEvent=False, then publish event is viewable
        history = view.getHistory(checkMayViewEvent=False)
        self.assertEqual(len(history), 3)
        actions = [event['action'] for event in history]
        self.assertTrue('publish' in actions)
        self.assertTrue(u'Edited' in actions)
        # cleanUp zmcl.load_config because it impact other tests
        zcml.cleanUp()

    def test_showRevisionInfos(self):
        """Test the showRevisionInfos method."""
        pr = api.portal.get_tool('portal_repository')
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')

        # As the type of doc is a versionable content, showRevisionInfo should
        # return True.
        self.assertTrue(view.showRevisionInfos())
        # Remove the type "Document" from the versionable content.
        pr.setVersionableContentType([u'ATDocument', u'ATNewsItem', u'Event', u'Link', u'News Item'])
        # Now showRevisionInfos shoud return False.
        self.assertFalse(view.showRevisionInfos())

    def test_renderComments(self):
        """Test the renderComments method."""
        # render comments will first try to translate the comments
        # then turn it from 'text/plain' to 'text/html'
        view = getMultiAdapter((self.doc, self.portal.REQUEST), name='contenthistory')
        view.histories_to_handle = (u'workflow', )

        # translated
        # do a transition with a comment that will be translatable
        self.wft.doActionFor(self.doc, 'publish', comment='data_change')
        last_event = view.getHistory()[0]
        self.assertEquals(view.renderComments(last_event), u'<p>Data change</p>')

        # turned from text/plain to text/html
        # do a transition with a comment that will be translatable
        self.wft.doActionFor(self.doc, 'retract', comment='Custom comments not translatable.\nAnd one additional line.')
        last_event = view.getHistory()[0]
        self.assertEquals(view.renderComments(last_event),
                          u'<p>Custom comments not translatable.<br />And one additional line.</p>')



# -*- coding: utf-8 -*-

from DateTime import DateTime
from imio import history as imio_history
from imio.history.interfaces import IImioHistory
from imio.history.testing import IntegrationTestCase
from imio.history.utils import add_event_to_history
from imio.history.utils import get_all_history_attr
from imio.history.utils import getLastAction
from imio.history.utils import getLastWFAction
from imio.history.utils import getPreviousEvent
from plone import api
from plone.memoize.instance import Memojito
from Products.Five import zcml
from zope.component import getAdapter


class TestUtils(IntegrationTestCase):

    def test_getPreviousEvent(self):
        """Test the getPreviousEvent method.
           It should return the previous event for a given event if it exists."""
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        adapter = getAdapter(doc, IImioHistory, 'workflow')
        history = adapter.getHistory()
        firstEvent = history[0]
        # this is the 'creation' event
        self.assertTrue(firstEvent['action'] is None)
        self.assertTrue(getPreviousEvent(doc, firstEvent) is None)

        # now publish the doc so we have an new event in the workflow_history
        api.content.transition(doc, 'publish', comment='My comment')
        # clean memoize
        getattr(adapter, Memojito.propname).clear()
        history = adapter.getHistory()
        lastEvent = history[-1]
        self.assertEqual(lastEvent['action'], 'publish')
        self.assertEqual(getPreviousEvent(doc, lastEvent), firstEvent)

        # if the event is not found, None is returned
        wrongEvent = {'action': 'wrong',
                      'review_state': 'wrong',
                      'comments': 'My wrong comment',
                      'actor': 'wrong',
                      'time': DateTime('2015/01/01 13:30:0.0 GMT+2')}
        self.assertTrue(getPreviousEvent(doc, wrongEvent) is None)

    def test_getLastAction(self):
        """Test the utils.getLastAction method.
           It should return the action passed in parameter for the given history name."""
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        # publish the doc so we have an new event in the workflow_history
        api.content.transition(doc, 'publish', comment='First publication comment')
        adapter = getAdapter(doc, IImioHistory, 'workflow')
        self.assertEqual(getLastAction(adapter)['action'], 'publish')
        # same as getting action with that name
        publish_action = getLastAction(adapter, action='publish')
        self.assertEqual(publish_action['action'], 'publish')
        self.assertEqual(publish_action['comments'], 'First publication comment')

        # publish again, check that we correctly get last action
        api.content.transition(doc, 'retract')
        api.content.transition(doc, 'publish', comment='Second publication comment')
        # clean memoize
        getattr(adapter, Memojito.propname).clear()
        publish_action = getLastAction(adapter, action='publish')
        self.assertEqual(publish_action['action'], 'publish')
        self.assertEqual(publish_action['comments'], 'Second publication comment')

        # the creation event is stored with a None action
        self.assertEqual(getLastAction(adapter, action=None)['review_state'], 'private')

        # if action not found, None is returned
        self.assertIsNone(getLastAction(adapter, action='unknown_action'))

    def test_getLastAction_history_empty(self):
        """Does not breaks and returns None if history empty."""
        adapter = getAdapter(self.portal.folder, IImioHistory, 'revision')
        self.assertIsNone(getLastAction(adapter))

    def test_getLastAction_checkMayView(self):
        """Check checkMayViewEvent/checkMayViewComment, default implementation
           returns True."""
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        # publish the doc so we have an new event in the workflow_history
        api.content.transition(doc, 'publish', comment='First publication comment')
        adapter = getAdapter(doc, IImioHistory, 'workflow')
        self.assertEqual(getLastAction(
            adapter, checkMayViewEvent=False, checkMayViewComment=False)['action'],
            'publish')
        self.assertEqual(getLastAction(
            adapter, checkMayViewEvent=True, checkMayViewComment=True)['action'],
            'publish')
        # enable testing-adapter.zcml, the 'publish' actions will
        # not be viewable anymore, as well as revisions
        zcml.load_config('testing-adapter.zcml', imio_history)
        self.request.set('hide_wf_history_event', True)
        self.request.set('hide_wf_history_comment', True)
        adapter = getAdapter(doc, IImioHistory, 'workflow')
        # checkMayViewEvent=False, checkMayViewComment=False
        res = getLastAction(adapter, checkMayViewEvent=False, checkMayViewComment=False)
        self.assertEqual(res['action'], 'publish')
        self.assertEqual(res['comments'], 'First publication comment')
        # checkMayViewEvent=True, checkMayViewComment=True
        self.assertIsNone(getLastAction(adapter, checkMayViewEvent=True, checkMayViewComment=True))
        # checkMayViewEvent=False, checkMayViewComment=True
        res = getLastAction(adapter, checkMayViewEvent=False, checkMayViewComment=True)
        self.assertEqual(res['action'], 'publish')
        self.assertEqual(res['comments'], adapter.comment_not_viewable_value)

    def test_getLastWFAction(self):
        """Test the utils.getLastWFAction method.
           It should return the action passed in parameter for the workflow_history.
           It is a shortcut using utils.getLastAction."""
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        # 'before_last' action is None
        self.assertIsNone(getLastWFAction(doc, transition='before_last'))
        # publish the doc so we have an new event in the workflow_history
        api.content.transition(doc, 'publish', comment='Publication comment')
        self.assertEqual(getLastWFAction(doc)['action'], 'publish')
        # same as getting action with that name
        publish_action = getLastWFAction(doc, transition='publish')
        self.assertEqual(publish_action['action'], 'publish')
        self.assertEqual(publish_action['comments'], 'Publication comment')
        self.assertEqual(getLastWFAction(doc, transition='before_last')['review_state'], 'private')

    def test_add_event_to_history(self):
        """Add an event to an history following an action."""
        folder = self.portal.folder
        # if history attr does not exist, it is created
        DUMMY_HISTORY_ATTR = 'dummy_history'
        self.assertFalse(hasattr(folder, DUMMY_HISTORY_ATTR))

        # action1 with default values
        add_event_to_history(folder, DUMMY_HISTORY_ATTR, 'action1')
        added_action1 = getattr(folder, DUMMY_HISTORY_ATTR)[0]
        self.assertEqual(added_action1['action'], 'action1')
        # current user id in actor
        self.assertEqual(added_action1['actor'], 'test_user_1_')
        self.assertEqual(added_action1['actor'], api.user.get_current().getId())
        self.assertEqual(added_action1['comments'], '')
        self.assertTrue(isinstance(added_action1['time'], DateTime))

        # action2 with parameters
        new_user = api.user.create(username='dummy_user', email='dummy@user.org')
        add_event_to_history(folder,
                             DUMMY_HISTORY_ATTR,
                             'action2',
                             actor=new_user,
                             time=DateTime('2018/01/12'),
                             comments=u'My comments',
                             extra_infos={'dummy_info1': u'Information 1',
                                          'dummy_info2': u'Information 2'})
        added_action2 = getattr(folder, DUMMY_HISTORY_ATTR)[1]
        self.assertEqual(added_action2['action'], 'action2')
        self.assertEqual(added_action2['actor'], new_user.getId())
        self.assertEqual(added_action2['comments'], u'My comments')
        self.assertEqual(added_action2['time'], DateTime('2018/01/12'))
        # extra infos
        self.assertEqual(added_action2['dummy_info1'], u'Information 1')
        self.assertEqual(added_action2['dummy_info2'], u'Information 2')

    def test_get_all_history_attr(self):
        """Get every coccurence of a given attr_name in a history."""
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        api.content.transition(doc, 'publish', comment='Publication comment')
        api.content.transition(doc, 'retract')
        self.assertEqual(get_all_history_attr(doc), [None, 'publish', 'retract'])
        self.assertEqual(get_all_history_attr(doc, attr_name='review_state'),
                         ['private', 'published', 'private'])
        self.assertEqual(get_all_history_attr(doc, attr_name='actor'),
                         ['test_user_1_', 'test_user_1_', 'test_user_1_'])
        self.assertEqual(get_all_history_attr(doc, attr_name='comments'),
                         ['', 'Publication comment', ''])



# -*- coding: utf-8 -*-
from imio.history.config import HISTORY_REVISION_NOT_VIEWABLE
from imio.history.interfaces import IImioHistory
from plone import api
from plone.app.layout.viewlets.content import ContentHistoryView
from plone.app.layout.viewlets.content import DocumentBylineViewlet
from plone.memoize.view import memoize
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getAdapter
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.i18n import translate


class IHDocumentBylineViewlet(DocumentBylineViewlet):
    """Overrides the DocumentBylineViewlet."""

    index = ViewPageTemplateFile("templates/document_byline.pt")

    def show_history(self):
        """Rely on contenthistory.show_history."""
        contenthistory = getMultiAdapter(
            (self.context, self.request), name='contenthistory')
        res = contenthistory.show_history()
        if res:
            # do not show a link to the history if we are displaying something in
            # an overlay because history is displayed in an overlay and it does not work...
            if 'ajax_load' in self.request:
                res = False
        return res

    def highlight_history_link(self):
        """
          If a comment was added to last event of the object history,
          we highlight the link (set a css class on it) so user eye is drawn to it.
        """
        adapter = getAdapter(self.context, IImioHistory, 'workflow')
        return adapter.historyLastEventHasComments()


class IHContentHistoryView(ContentHistoryView):
    '''
      Overrides the ContentHistoryView template to use our own.
      We want to display the content_history as a table.
    '''
    histories_to_handle = (u'revision', u'workflow')
    index = ViewPageTemplateFile("templates/content_history.pt")

    def getHistory(self, checkMayViewEvent=True, checkMayViewComment=True):
        """Get the history for current object.
           Merge workflow history with content history and sort by time."""
        history = []
        history_adapters = getAdapters((self.context,), IImioHistory)
        for adapter_name, adapter in history_adapters:
            # for now, specifically limit display to u'revision' and u'workflow'
            if adapter_name in self.histories_to_handle:
                history.extend(adapter.getHistory(
                    checkMayViewEvent=checkMayViewEvent,
                    checkMayViewComment=checkMayViewComment))

        if not history:
            return []

        history.sort(key=lambda x: x["time"], reverse=True)
        return history

    def getTransitionTitle(self, transitionName):
        """
          Given a p_transitionName, return the defined title in portal_workflow
          as it is what is really displayed in the template.
        """
        currentWF = self._getCurrentContextWorkflow()
        if currentWF and transitionName in currentWF.transitions and \
           currentWF.transitions[transitionName].title:
            return currentWF.transitions[transitionName].title
        else:
            return transitionName

    def _extra_render_comments_mapping(self):
        """ """
        return {}

    def renderComments(self, event):
        """
          Render comments correctly as it is 'plain/text' and we want 'text/html'.
        """
        # prepare some data passed to translate as mappings
        mapping = event.copy()
        mapping['event_time'] = int(event['time'])
        mapping['url'] = self.context.absolute_url()
        mapping.update(self._extra_render_comments_mapping())
        # try to translate comments before it is turned into text/html
        translated = translate(
            safe_unicode(event['comments']),
            mapping=mapping,
            domain='imio.history',
            context=self.request)
        transformsTool = api.portal.get_tool('portal_transforms')
        data = transformsTool.convertTo('text/x-html-safe', translated)
        return data.getData()

    @memoize
    def _getCurrentContextWorkflow(self):
        """
          Return currently used workflow.
        """
        wfTool = getToolByName(self.context, 'portal_workflow')
        workflows = wfTool.getWorkflowsFor(self.context)
        return workflows and workflows[0] or None

    def showColors(self):
        """
          Colorize transition name?
        """
        return True

    def show_history(self):
        """
          Show the history?  This is a common method used by :
          - the view (@@historyview);
          - the viewlet (imio.history.documentbyline);
          - imio.actionspanel history action icon.
          Originally, the history is shown to people having the
          'CMFEditions: Access previous versions' permission, here
          we want everybody than can acces the object to see the history...
        """
        return True

    def showRevisionInfos(self):
        """Return True if the type of the context is versioned. """
        pr = getToolByName(self.context, 'portal_repository')
        if self.context.portal_type in pr.getVersionableContentTypes():
            return True
        else:
            return False

    def versionIsViewable(self, event):
        """
          Check if version we want to show is viewable.
        """
        return not bool(event['comments'] == HISTORY_REVISION_NOT_VIEWABLE)

    def renderCustomJS(self):
        """ """
        return '<script></script>'


class IHVersionPreviewView(BrowserView):
    """Makes it possible to display a preview of a given version."""

    def __init__(self, context, request):
        """ """
        super(IHVersionPreviewView, self).__init__(context, request)
        self.portal = getToolByName(self.context, 'portal_url').getPortalObject()
        self.portal_url = self.portal.absolute_url()

    def __call__(self, version_id):
        pr = getToolByName(self.context, 'portal_repository')
        self.versioned_object = pr.retrieve(self.context, version_id).object
        return super(IHVersionPreviewView, self).__call__()



# -*- coding: utf-8 -*-

from imio.history.interfaces import IImioHistory
from imio.history.testing import IntegrationTestCase
from plone import api
from plone.app.testing import TEST_USER_ID
from zope.component import getAdapter


class TestImioWFHistoryAdapter(IntegrationTestCase):

    """Test ImioWFHistoryAdapter."""

    def setUp(self):
        super(TestImioWFHistoryAdapter, self).setUp()
        doc = api.content.create(type='Document',
                                 id='doc',
                                 container=self.portal)
        self.doc = doc

    def test_getHistory(self):
        adapter = getAdapter(self.doc, IImioHistory, 'workflow')
        self.wft.doActionFor(self.doc, 'publish')
        history = adapter.getHistory()
        self.assertEqual(len(history), 2)
        # creation event
        creation_event = history[0]
        self.assertEqual(creation_event['type'], 'workflow')
        self.assertEqual(creation_event['review_state'], 'private')
        self.assertEqual(creation_event['actor'], TEST_USER_ID)
        self.assertIsNone(creation_event['action'])
        # creation event
        publish_event = history[1]
        self.assertEqual(publish_event['type'], 'workflow')
        self.assertEqual(publish_event['review_state'], 'published')
        self.assertEqual(publish_event['actor'], TEST_USER_ID)
        self.assertEqual(publish_event['action'], 'publish')

    def test_getHistoryWithPreviousReviewState(self):
        """ """
        adapter = getAdapter(self.doc, IImioHistory, 'workflow')
        self.wft.doActionFor(self.doc, 'publish')
        self.assertFalse(adapter.include_previous_review_state)
        adapter.include_previous_review_state = True
        history = adapter.getHistory()
        # creation event
        creation_event = history[0]
        self.assertEqual(creation_event['type'], 'workflow')
        self.assertEqual(creation_event['review_state'], 'private')
        self.assertEqual(creation_event['actor'], TEST_USER_ID)
        self.assertIsNone(creation_event['action'])
        self.assertIsNone(creation_event['previous_review_state'])
        # creation event
        publish_event = history[1]
        self.assertEqual(publish_event['type'], 'workflow')
        self.assertEqual(publish_event['review_state'], 'published')
        self.assertEqual(publish_event['actor'], TEST_USER_ID)
        self.assertEqual(publish_event['action'], 'publish')
        self.assertEqual(publish_event['previous_review_state'], 'private')



# -*- coding: utf-8 -*-

from DateTime import DateTime
from imio.history.interfaces import IImioHistory
from persistent.list import PersistentList
from plone import api
from Products.CMFPlone.utils import base_hasattr
from zope.component import getAdapter


def _check_may_view(event, adapter, checkMayViewEvent=False, checkMayViewComment=False):
    if event is not None:
        if checkMayViewEvent:
            if not adapter.mayViewEvent(event):
                event = None
        if checkMayViewComment and event is not None:
            if not adapter.mayViewComment(event):
                event['comments'] = adapter.comment_not_viewable_value
    return event


def getPreviousEvent(obj, event, checkMayViewEvent=False, checkMayViewComment=False):
    '''Returns the previous event found in the history for the given p_event
       on p_obj if p_event is found.  p_checkMayView is passed to IImioHistory.getHistory
       and will enable/disable event's comments viewability check.'''

    adapter = getAdapter(obj, IImioHistory, 'workflow')
    # for performance, checkMayViewEvent and checkMayViewComment only on found event
    history = adapter.getHistory(
        checkMayViewEvent=False, checkMayViewComment=False)
    res = None
    if event in history and history.index(event) > 0:
        res = history[history.index(event) - 1]

    return _check_may_view(res, adapter, checkMayViewEvent, checkMayViewComment)


def getLastAction(adapter, action='last', checkMayViewEvent=False, checkMayViewComment=False):
    '''Returns, from the p_history_name of p_adapter, the last occurence of p_action.
       Default p_action is 'last' because we also want to be able to get
       an action that is 'None' in a particular p_history_name.'''

    # for performance, checkMayViewEvent and checkMayViewComment only on found event
    history = adapter.getHistory(
        checkMayViewEvent=False, checkMayViewComment=False)

    res = None
    if action == 'last':
        # do not break if history is empty
        res = history and history[-1] or None
    elif action == 'before_last':
        # do not break if history empty or only contains one single event
        res = len(history) > 1 and history[-2] or None
    else:
        i = len(history) - 1
        while i >= 0:
            event = history[i]
            if isinstance(action, basestring):
                condition = event['action'] == action
            elif action is None:
                condition = event['action'] is None
            else:
                condition = event['action'] in action
            if condition:
                res = event
                break
            i -= 1

    return _check_may_view(res, adapter, checkMayViewEvent, checkMayViewComment)


def getLastWFAction(obj, transition='last', checkMayViewEvent=False, checkMayViewComment=False):
    '''Helper to get last p_transition workflow_history event.
       By default, security checks are not done (checkMayViewEvent=False, checkMayViewComment=False).'''
    adapter = getAdapter(obj, IImioHistory, 'workflow')
    last_wf_action = getLastAction(adapter,
                                   action=transition,
                                   checkMayViewEvent=checkMayViewEvent,
                                   checkMayViewComment=checkMayViewComment)
    return last_wf_action


def get_all_history_attr(obj,
                         attr_name='action',
                         history_name='workflow',
                         checkMayViewEvent=False,
                         checkMayViewComment=False):
    '''Return a list of every p_attr_name for p_history_name of p_obj.'''
    adapter = getAdapter(obj, IImioHistory, history_name)
    res = [event[attr_name] for event in adapter.getHistory(checkMayViewEvent, checkMayViewComment)]
    return res


def add_event_to_history(obj, history_attr, action, actor=None, time=None, comments=u'', extra_infos={}):
    '''This is an helper method to add an entry to an history.'''
    if not base_hasattr(obj, history_attr):
        setattr(obj, history_attr, PersistentList())
    history_data = {'action': action,
                    'actor': actor and actor.getId() or api.user.get_current().getId(),
                    'time': time or DateTime(),
                    'comments': comments}
    history_data.update(extra_infos)
    getattr(obj, history_attr).append(history_data.copy())
